#!/bin/sh
#
# Create and deploy an NPM package.
#
# Start it in the npm packaging subdirectory of your project.
#

echo "Running in directory "$(/bin/pwd)

dry=0
yes=0

#### local config
srcfolder="../src"
targetfolder="../target/npm-package"

#### Remote config
category=
sshserver="clazzesnpm@www.clazzes.org"
webhost="npm.clazzes.org"

usage() {

    echo "Usage: /usr/share/pba-ci/sh/create-npm-package.sh <options>"
    echo "Options are:"
    echo "  --help|-h"
    echo "      Print this help."
    echo "  --yes"
    echo "      Perform with no regrets, don't ask the user on the console to perform."
    echo "  --dry"
    echo "      Perform dry run, do not scp, do not commit or push anything."
    echo "  --src <folder>"
    echo "      Source folder with package.json, defaults to '../src'."
    echo "  --target <folder>"
    echo "      Target folder with package.json, defaults to '../target/npm-package'."
    echo "  --ssh-server <server>"
    echo "      SSH server used for uploads, defaults to 'clazzesnpm@www.clazzes.org'."
    echo "  --web-host <host>"
    echo "      DNS name of target web host, defaults to 'npm.clazzes.org'."
    echo "  --category <category>"
    echo "      Subdirectory in Web host to place artifacts, must be specified."
    echo "      Should be something like 'org/clazzes/dojo'."
    echo ""
}

while test $# -gt 0
do
  key="$1"

  case $key in
      --help|-h)
          usage
          exit 0
          ;;
      --dry)
          dry=1
          echo "Performing dry run, revert with"
          echo "  git checkout src/package.json"
          echo ""
          shift 1
          ;;
      --yes)
          yes=1
          echo "Performing in non-interactive mode with no regrets."
          shift 1
          ;;
      --ssh-server)
          sshserver="$2"
          shift 2
          ;;
      --web-host)
          webhost="$2"
          shift 2
          ;;
      --src)
          srcfolder="$2"
          shift 2
          ;;
      --target)
          targetfolder="$2"
          shift 2
          ;;
      --category)
          category="$2"
          shift 2
          ;;
      *)  # unknown option
          echo "Error: Unknown option [$1]."
          usage
          exit 64
          ;;
  esac
done

if test "$category" = ""
then
    dry=1
    echo "Error: No category given, exiting."
    usage
    exit 64
fi

if test -z "$sshserver" -o -z "$webhost" -o -z "$targetfolder" -o -z "$srcfolder" 
then
    dry=1
    echo "Error: One of the default values has been overwritten with an empty value."
    usage
    exit 64
fi

perform() {

    echo $*
    if test $dry -eq 0
    then
        "$@"
    fi
}

#### derived config
package_config="${targetfolder}/package.json"
scp_path="/var/www/htdocs/${webhost}/${category}"


version=$(sed -E 's/^ *"version": *"(.*)" *, *$/\1/p;d' ${srcfolder}/package.json)
name=$(sed -E 's/^ *"name": *"(.*)" *, *$/\1/p;d' ${srcfolder}/package.json)

echo "Package version is $version"
echo "Package name is $name"

if echo $version | grep -E -v -q -- '\.[0-9]+-SNAPSHOT$'
then
    echo "Error: Version is not ending with -SNAPSHOT."
    exit 1
fi

relversion=$(echo $version | sed -E 's/-SNAPSHOT$//')
minorversion=$(echo $version | sed -E 's/^.*\.([0-9]+)-SNAPSHOT$/\1/p;d')

nextminor=$(expr $minorversion + 1)

nextversion=$(echo $version | sed -E 's/(^.*)\.[0-9]+-SNAPSHOT$/\1.'$nextminor'-SNAPSHOT/p;d')

echo "Packaging $name for version $relversion"
echo "Next version will be $nextversion"

if test $yes -eq 0
then
    read -p "Do you want to build version $relversion? " REPLY
    if echo $REPLY | grep -q -v '^[YyJj]$'
    then
        exit 42
    fi
fi

# set release version
mv ${srcfolder}/package.json ${srcfolder}/package.json~
sed -E 's/^ *"version": *"(.*)" *, *$/  "version": "'$relversion'",/' < ${srcfolder}/package.json~ > ${srcfolder}/package.json

                                                                                         
######## Compiling

(cd $srcfolder && npm run clean)
(cd $srcfolder && npm ci)
(cd $srcfolder && npm run build)


######## Read from package.json

if ! test -f "${package_config}"; then
  >&2 echo "$package_config does not exist"
  exit 1;
fi

targetversion=$(sed -E 's/^ *"version": *"(.*)" *, *$/\1/p;d' ${package_config})

if test "$targetversion" != "$relversion"
then
    echo "Version $targetversion of target package.lock is not $relversion"
    exit 2
fi

echo "Packaging $name for version $relversion"


######################### Preparing & Modifying

#mkdir -p $tmpfolder
#rm -r $tmpfolder
#cp -r $targetfolder $tmpfolder
#cp $targetfolder/../*.d.ts $tmpfolder

######################### Creating package

# if we would like to move package.json and README.md into different folders
#cp $package_config $tmpfolder
#cp "../README.md" $tmpfolder

TAR_FILENAME="$name-$relversion.tar.gz"

tar --create --file "${TAR_FILENAME}" --gzip --exclude=node_modules --recursion -C $targetfolder .
echo "\e[32mCreated\e[m '$TAR_FILENAME' locally"


######################### Deployment

if test $yes -eq 0
then
    read -p "Do you want to deploy version $relversion? " REPLY
    if echo $REPLY | grep -q -v '^[YyJj]$'
    then
        exit 42
    fi
fi

# Make dir if necessary
perform ssh ${sshserver} mkdir -pv ${scp_path} || exit 12

# Is the tarball already existing? Fail...
if ssh ${sshserver} stat "${scp_path}/$TAR_FILENAME" \> /dev/null 2\>\&1
then
  echo -e "tarball already exists on server -\e[31m aborting \e[m"
  exit 11
fi

echo "File does not exist -\e[34m uploading\e[m"
perform scp "${TAR_FILENAME}" "${sshserver}:${scp_path}/" || exit 13

# ggfs noch zur sicherheit
perform ssh ${sshserver} chmod -R ugo+rx ${scp_path}/ || exit 14

echo "\e[32mUploaded \e[m'$TAR_FILENAME'"
echo "\e[32mhttps://${webhost}/${category}/$TAR_FILENAME\e[m"

# commit release version
perform git add ${srcfolder}/package.json
perform git commit -m "Released $name-$relversion"
perform git push

perform git tag -a $relversion -m "$name-$relversion release"
perform git push origin $relversion

# set next version
mv ${srcfolder}/package.json ${srcfolder}/package.json~
sed -E 's/^ *"version": *"(.*)" *, *$/  "version": "'$nextversion'",/' < ${srcfolder}/package.json~ > ${srcfolder}/package.json

perform git add ${srcfolder}/package.json
perform git commit -m "Set version to $name-$nextversion"
perform git push

if test $dry -ne 0
then
    echo "\e[32mPerformed dry run, revert with\e[m"
    echo "  git checkout src/package.json"
    echo ""
fi
